type SerializableValue = Record<string, unknown> | string | number | boolean | null | SerializableObject | Array<SerializableValue>;
type SerializableObject = {
    [key: string]: SerializableValue;
};
type EncoderFunction<Source> = (value: Source, encoder: (value: SerializableValue) => Uint8Array) => Uint8Array;
type DecoderFunction<Result> = (source: Uint8Array, decoder: (value: Uint8Array) => SerializableValue) => Result;
interface Encoder<T> {
    test: (v: unknown) => boolean;
    encode: EncoderFunction<T>;
    decode: DecoderFunction<T>;
}
type EncoderList = Array<[number, Encoder<any>]>;
/**
 * Serializes the content of the object
 * @param value
 */
declare const serialize: (val: SerializableValue) => Uint8Array;
/**
 * Deserializes a serialized object
 * @param data
 */
declare const deserialize: (val: Uint8Array) => SerializableValue;
type WrappedEncoder = {
    serialize: typeof serialize;
    deserialize: typeof deserialize;
};
/**
 * Injects custom-encoders
 */
declare const use: (extra: EncoderList) => WrappedEncoder;
declare const utils: {
    pack: (content: Uint8Array) => Uint8Array;
    unpack: (content: Uint8Array, offset?: number) => [Uint8Array, number];
    concat: (...arrays: Uint8Array[]) => Uint8Array;
};
declare const version: string;
export { SerializableValue, SerializableObject, EncoderFunction, DecoderFunction, Encoder, EncoderList, serialize, deserialize, WrappedEncoder, use, utils, version };
